package fr.uca.iut.urbandictionarylight.data

import android.app.Application
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import fr.uca.iut.urbandictionarylight.model.Definition
import fr.uca.iut.urbandictionarylight.model.Entry
import fr.uca.iut.urbandictionarylight.model.EntryWithDefinitions

private const val DB_FILE = "u_d_light.db"

@Database(entities = [Entry::class, Definition::class], version = 1)
abstract class UrbanDictionaryDatabase : RoomDatabase() {

    abstract fun entryDao(): EntryDao
    abstract fun definitionDao(): DefinitionDao

    companion object {
        private lateinit var application: Application

        @Volatile
        private var instance: UrbanDictionaryDatabase? = null

        fun getInstance(): UrbanDictionaryDatabase {
            if (Companion::application.isInitialized) {
                if (instance == null)
                    synchronized(this) {
                        if (instance == null) {
                            instance = Room.databaseBuilder(
                                application.applicationContext,
                                UrbanDictionaryDatabase::class.java,
                                DB_FILE
                            )
                                .allowMainThreadQueries()
                                .build()

                            instance?.entryDao()?.let {
                                if (it.queryAll().isEmpty()) emptyDatabaseStub(it)
                            }
                        }
                    }
                return instance!!
            } else
                throw RuntimeException("the database must be initialized first")
        }

        //????
        @Synchronized
        fun initialize(app: Application) {
            if (Companion::application.isInitialized)
                throw RuntimeException("the database must not be initialized twice")

            application = app
        }


        private fun emptyDatabaseStub(
            entryDao: EntryDao,
            definitionDao: DefinitionDao
        ): () -> Unit = {

            val def1 = Definition("bleep", "bleep blap bloop", 1)
            val def2 = Definition("blarp", "bleeeep blap bloop", 7)
            val def3 = Definition("blep", "bleep blap blooop", 4)
            val def4 = Definition("blupo", "bleep blaap bloop", 56)

            with(definitionDao) {
                insert(def1)
                insert(def2)
                insert(def3)
                insert(def4)
            }
            with(entryDao) {
                insert(
                    EntryWithDefinitions(
                        Entry("shrimping"),
                        listOf(def1, def4)
                    )
                )
                insert(
                    EntryWithDefinitions(
                        Entry("blunderkegel"),
                        listOf(def2)
                    )
                )
                insert(
                    EntryWithDefinitions(
                        Entry("something horrible and dumb"),
                        listOf(def3)
                    )
                )
            }
        }
    }
}
